@@ -16,41 +16,73 @@ from wheatley.rhythm import Rhythm, RegressionRhythm, WaitForUserRhythm
 from wheatley.tower import RingingRoomTower
 from wheatley.bot import Bot
 from wheatley.page_parser import get_load_balancing_url, TowerNotFoundError, InvalidURLError
-from wheatley.parsing import parse_peal_speed, PealSpeedParseError, parse_call
+from wheatley.parsing import (
+    parse_peal_speed,
+    PealSpeedParseError,
+    parse_call,
+    parse_start_row,
+    StartRowParseError,
+    parse_place_notation,
+    PlaceNotationError,
+)
 
 from wheatley.row_generation import RowGenerator, ComplibCompositionGenerator
 from wheatley.row_generation import MethodPlaceNotationGenerator
 from wheatley.row_generation.complib_composition_generator import PrivateCompError, InvalidCompError
-from wheatley.row_generation.method_place_notation_generator import MethodNotFoundError, \
-                                                                    generator_from_special_title
+from wheatley.row_generation.method_place_notation_generator import (
+    MethodNotFoundError,
+    generator_from_special_title,
+)
 from wheatley.row_generation.place_holder_generator import PlaceHolderGenerator
+from wheatley.row_generation.place_notation_generator import PlaceNotationGenerator
 
 
 def create_row_generator(args: argparse.Namespace) -> RowGenerator:
-    """ Generates a row generator according to the given CLI arguments. """
-    if "comp" in args and args.comp is not None:
+    """Generates a row generator according to the given CLI arguments."""
+    if "comp" in args and args.comp is not None and "start_row" in args and args.start_row is not None:
+        sys.exit("You may not specify a custom start row with a composition")
+    elif "comp" in args and args.comp is not None:
         try:
             return ComplibCompositionGenerator.from_arg(args.comp)
         except (PrivateCompError, InvalidCompError) as e:
             sys.exit(f"Bad value for '--comp': {e}")
-    elif "method" in args:
+    elif "method" in args and args.method is not None:
         try:
-            return generator_from_special_title(args.method) or \
-                      MethodPlaceNotationGenerator(
-                          args.method,
-                          parse_call(args.bob),
-                          parse_call(args.single),
-                          args.start_index
-                      )
+            return generator_from_special_title(args.method, args.start_row) or MethodPlaceNotationGenerator(
+                args.method,
+                parse_call(args.bob),
+                parse_call(args.single),
+                args.start_row,
+                args.start_index,
+            )
         except MethodNotFoundError as e:
             sys.exit(f"Bad value for '--method': {e}")
-    else:
-        assert False, "This shouldn't be possible because one of --method and --comp should always be defined"
+    elif "place_notation" in args and args.place_notation is not None:
+        try:
+            stage, place_notation = parse_place_notation(args.place_notation)
+            return PlaceNotationGenerator(
+                stage,
+                place_notation,
+                parse_call(args.bob),
+                parse_call(args.single),
+                args.start_index,
+                args.start_row,
+            )
+        except PlaceNotationError as e:
+            sys.exit(f"Bad value for '--place-notation': {e}")
+
+    raise AssertionError("One of --method, --comp or --place-notation should always be defined")
 
 
-def create_rhythm(peal_speed: int, inertia: float, max_bells_in_dataset: int, handstroke_gap: float,
-                  use_wait: bool, initial_inertia: float=0) -> Rhythm:
-    """ Generates a rhythm object according to the given CLI arguments. """
+def create_rhythm(
+    peal_speed: int,
+    inertia: float,
+    max_bells_in_dataset: int,
+    handstroke_gap: float,
+    use_wait: bool,
+    initial_inertia: float = 0,
+) -> Rhythm:
+    """Generates a rhythm object according to the given CLI arguments."""
     # Sets the minimum number of bells that Wheatley will use in order to deduce a rhythm.  Setting this to
     # larger numbers will make Wheatley more stable during the pull-off, whereas smaller numbers will make
     # Wheatley more sensitive to user's pull-off speed.  If this is larger than `--max_bells_in_dataset`,
@@ -66,7 +98,7 @@ def create_rhythm(peal_speed: int, inertia: float, max_bells_in_dataset: int, ha
         peal_speed=peal_speed,
         min_bells_in_dataset=min_bells_in_dataset,
         max_bells_in_dataset=max_bells_in_dataset,
-        initial_inertia=initial_inertia
+        initial_inertia=initial_inertia,
     )
 
     if use_wait:
@@ -90,7 +122,7 @@ def get_version_number() -> str:
 
 
 def configure_logging(verbose: bool, quietness: int) -> None:
-    """ Sets up the logging for Wheatley. """
+    """Sets up the logging for Wheatley."""
     logging.basicConfig(level=logging.WARNING)
 
     # Verboseness is a numerical represntation of the log level, with the following cases:
@@ -135,36 +167,36 @@ def server_main(override_args: Optional[List[str]], stop_on_join_tower: bool) ->
         "room_id",
         type=int,
         help="The numerical ID of the tower to join, represented as a row on 9 bells, \
-              e.g. 763451928."
+              e.g. 763451928.",
     )
     parser.add_argument(
-        "-p", "--port",
+        "-p",
+        "--port",
         type=int,
-        help="The port of the SocketIO server (which must be hosted on localhost)."
+        help="The port of the SocketIO server (which must be hosted on localhost).",
     )
     parser.add_argument(
-        "-l", "--look-to-time",
+        "-l",
+        "--look-to-time",
         type=float,
         help="Set to the time when 'Look to' was called if Wheatley was spawned because 'look to' was \
-              called and Wheatley is needed."
+              called and Wheatley is needed.",
     )
+    parser.add_argument("-i", "--id", type=int, help="The instance ID of this Wheatley process.")
     parser.add_argument(
-        "-i", "--id",
-        type=int,
-        help="The instance ID of this Wheatley process."
-    )
-    parser.add_argument(
-        "-v", "--verbose",
+        "-v",
+        "--verbose",
         action="count",
         default=0,
-        help="Makes Wheatley print more (DEBUG) output."
+        help="Makes Wheatley print more (DEBUG) output.",
     )
     parser.add_argument(
-        "-q", "--quiet",
+        "-q",
+        "--quiet",
         action="count",
         default=0,
         help="Makes Wheatley print less output.  `-q` only prints WARNINGs and ERRORs; `-qq` only prints \
-              ERRORs; `-qqq` prints nothing."
+              ERRORs; `-qqq` prints nothing.",
     )
 
     # Parse arguments
@@ -192,10 +224,19 @@ def server_main(override_args: Optional[List[str]], stop_on_join_tower: bool) ->
     tower_url = "http://127.0.0.1:" + str(args.port)
 
     tower = RingingRoomTower(args.room_id, tower_url)
-    rhythm = create_rhythm(peal_speed, inertia, max_bells_in_dataset, handstroke_gap, use_wait,
-                           initial_inertia)
-    bot = Bot(tower, PlaceHolderGenerator(), use_up_down_in, stop_at_rounds, call_comps, rhythm,
-              user_name="Wheatley", server_instance_id=args.id)
+    rhythm = create_rhythm(
+        peal_speed, inertia, max_bells_in_dataset, handstroke_gap, use_wait, initial_inertia
+    )
+    bot = Bot(
+        tower,
+        PlaceHolderGenerator(),
+        use_up_down_in,
+        stop_at_rounds,
+        call_comps,
+        rhythm,
+        user_name="Wheatley",
+        server_instance_id=args.id,
+    )
 
     with tower:
         tower.wait_loaded()
@@ -221,9 +262,7 @@ def console_main(override_args: Optional[List[str]], stop_on_join_tower: bool) -
 
     # PARSE THE ARGUMENTS
 
-    parser = argparse.ArgumentParser(
-        description="A bot to fill in bells during ringingroom.com practices"
-    )
+    parser = argparse.ArgumentParser(description="A bot to fill in bells during ringingroom.com practices")
 
     # Tower arguments
     tower_group = parser.add_argument_group("Tower arguments")
@@ -231,36 +270,43 @@ def console_main(override_args: Optional[List[str]], stop_on_join_tower: bool) -
         "room_id",
         type=int,
         help="The numerical ID of the tower to join, represented as a row on 9 bells, \
-              e.g. 763451928."
+              e.g. 763451928.",
     )
     tower_group.add_argument(
         "--url",
         default="https://ringingroom.com",
         type=str,
-        help="The URL of the server to join (defaults to 'https://ringingroom.com')"
+        help="The URL of the server to join (defaults to 'https://ringingroom.com')",
     )
     tower_group.add_argument(
         "--name",
         default=None,
         type=str,
         help="If set, then Wheatley will ring bells assigned to the given name. \
-             When not set, Wheatley rings unassigned bells."
+             When not set, Wheatley rings unassigned bells.",
     )
 
     # Row generation arguments
     row_gen_group = parser.add_argument_group("Row generation arguments")
 
-    # An mutual exclusion group to disallow specifying both a method *and* a CompLib comp
-    comp_method_group = row_gen_group.add_mutually_exclusive_group(required=True)
-    comp_method_group.add_argument(
-        "-c", "--comp",
+    # An mutual exclusion group to disallow specifying more than one of a method,
+    # a CompLib comp or a place notation
+    comp_method_place_group = row_gen_group.add_mutually_exclusive_group(required=True)
+    comp_method_place_group.add_argument(
+        "-c",
+        "--comp",
         type=str,
-        help="The ID or URL of the complib composition you want to ring"
+        help="The ID or URL of the complib composition you want to ring. \
+        This can include a substituted method or access key query string",
     )
-    comp_method_group.add_argument(
-        "-m", "--method",
+    comp_method_place_group.add_argument(
+        "-m", "--method", type=str, help="The title of the method you want to ring"
+    )
+    comp_method_place_group.add_argument(
+        "-p",
+        "--place-notation",
         type=str,
-        help="The title of the method you want to ring"
+        help="The place notation description of the method you want to ring",
     )
 
     row_gen_group.add_argument(
@@ -272,7 +318,7 @@ def console_main(override_args: Optional[List[str]], stop_on_join_tower: bool) -
               "0:16" => 6ths place lead end bob.  "-1:3" or "-1:3.1" => a Grandsire Bob.  "20: 70" \
               => a 70 bob taking effect 20 changes into a lead (the Half Lead for Surprise Royal). \
               "20:7/0:4" => a 70 bob 20 changes into a lead and a 14 bob at the lead end. \
-              "3: 5/9: 5" => bobs in Stedman Triples.  Defaults to "14".'
+              "3: 5/9: 5" => bobs in Stedman Triples.  Defaults to "14".',
     )
     row_gen_group.add_argument(
         "-n",
@@ -282,110 +328,117 @@ def console_main(override_args: Optional[List[str]], stop_on_join_tower: bool) -
               Ringing Room.  These will by default happen at the lead end.  Examples: "1678" or \
               "0:1678" => 6ths place lead end single.  "-1:3.123" => a Grandsire Single. \
               "20: 7890" => a 7890 single taking effect 20 changes into a lead (the Half Lead for \
-              Surprise Royal). "3: 567/9: 567" => singles in Stedman Triples.  Defaults to "1234".'
+              Surprise Royal). "3: 567/9: 567" => singles in Stedman Triples.  Defaults to "1234".',
     )
     row_gen_group.add_argument(
         "--start-index",
         type=int,
         default=0,
         help="Determines which row of the lead Wheatley will start ringing.  This can be negative (so -1 \
-              would refer to the lead **end**).  Defaults to 0 (i.e. a standard start)."
+              would refer to the lead **end**).  Defaults to 0 (i.e. a standard start).",
     )
+    row_gen_group.add_argument("--start-row", type=str, help="Determines the initial row.")
     row_gen_group.add_argument(
-        "-u", "--use-up-down-in",
+        "-u",
+        "--use-up-down-in",
         action="store_true",
         help="If set, then the Wheatley will automatically go into changes after two rounds have been \
-              rung."
+              rung.",
     )
     row_gen_group.add_argument(
-        "-s", "--stop-at-rounds",
+        "-s",
+        "--stop-at-rounds",
         action="store_true",
-        help="If set, then Wheatley will stand his bells the first time rounds is reached."
+        help="If set, then Wheatley will stand his bells the first time rounds is reached.",
     )
     row_gen_group.add_argument(
-        "-H", "--handbell-style",
+        "-H",
+        "--handbell-style",
         action="store_true",
         help="If set, then Wheatley will ring 'handbell style', i.e. ringing two strokes of \
               rounds then straight into changes, and stopping at the first set of rounds. By \
               default, he will ring 'towerbell style', i.e. only taking instructions from the \
-              ringing-room calls. This is equivalent to using the '-us' flags."
+              ringing-room calls. This is equivalent to using the '-us' flags.",
     )
     row_gen_group.add_argument(
         "--no-calls",
         action="store_true",
-        help="If set, Wheatley will not call anything when ringing compositions."
+        help="If set, Wheatley will not call anything when ringing compositions.",
     )
 
     # Rhythm arguments
     rhythm_group = parser.add_argument_group("Rhythm arguments")
     rhythm_group.add_argument(
-        "-k", "--keep-going",
+        "-k",
+        "--keep-going",
         action="store_true",
         help="If set, Wheatley will not wait for users to ring - instead, he will push on with the \
-              rhythm."
+              rhythm.",
     )
     rhythm_group.add_argument(
-        "-w", "--wait",
+        "-w",
+        "--wait",
         action="store_true",
         help="Legacy parameter, which is now set by default. The previous default behaviour of not waiting \
-              can be set with '-k'/'--keep-going'."
+              can be set with '-k'/'--keep-going'.",
     )
     rhythm_group.add_argument(
-        "-I", "--inertia",
+        "-I",
+        "--inertia",
         type=float,
         default=0.5,
         help="Overrides Wheatley's 'inertia' - how much Wheatley will take other ringers' positions \
               into account when deciding when to ring.  0.0 means he will cling as closely as \
               possible to the current rhythm, 1.0 means that he will completely ignore the other \
-              ringers."
+              ringers.",
     )
     rhythm_group.add_argument(
-        "-S", "--peal-speed",
+        "-S",
+        "--peal-speed",
         default="2h58",
         help="Sets the default speed that Wheatley will ring (assuming a peal of 5040 changes), \
               though this will usually be adjusted by Wheatley whilst ringing to keep with other \
               ringers.  Example formatting: '3h4' = '3h4m' = '3h04m' = '3h04' = '184m' = '184'. \
-              Defaults to '2h58'."
+              Defaults to '2h58'.",
     )
     rhythm_group.add_argument(
-        "-G", "--handstroke-gap",
+        "-G",
+        "--handstroke-gap",
         type=float,
         default=1.0,
         help="Sets the handstroke gap as a factor of the space between two bells.  Defaults to \
-              '1.0'."
+              '1.0'.",
     )
     rhythm_group.add_argument(
-        "-X", "--max-bells-in-dataset",
+        "-X",
+        "--max-bells-in-dataset",
         type=int,
         default=15,
         help="Sets the maximum number of bells that Wheatley will store to determine the current \
               ringing speed.  If you make this larger, then he will be more consistent but less \
               quick to respond to changes in rhythm.  Setting both this and \
               --inertia to a very small values could result in Wheatley ringing ridiculously \
-              quickly.  Defaults to '15'."
+              quickly.  Defaults to '15'.",
     )
 
     # Misc arguments
+    parser.add_argument("--version", action="version", version=f"Wheatley {__version__}")
     parser.add_argument(
-        "--version",
-        action="version",
-        version=f"Wheatley {__version__}"
-    )
-    parser.add_argument(
-        "-v", "--verbose",
+        "-v",
+        "--verbose",
         action="count",
         default=0,
-        help="Makes Wheatley print more (DEBUG) output."
+        help="Makes Wheatley print more (DEBUG) output.",
     )
     parser.add_argument(
-        "-q", "--quiet",
+        "-q",
+        "--quiet",
         action="count",
         default=0,
         help="Makes Wheatley print less output.  `-q` only prints WARNINGs and ERRORs; `-qq` only prints \
-              ERRORs; `-qqq` prints nothing."
+              ERRORs; `-qqq` prints nothing.",
     )
 
-
     # Parse arguments
     # `[1:]` is apparently needed, because sys.argv[0] is the working file of the Python interpreter
     # which `parser.parse_args` does not want to see as an argument
@@ -405,6 +458,11 @@ def console_main(override_args: Optional[List[str]], stop_on_join_tower: bool) -
     except InvalidURLError as e:
         sys.exit(f"Bad value for '--url': {e}")
 
+    try:
+        parse_start_row(args.start_row)
+    except StartRowParseError as e:
+        sys.exit(f"{e}")
+
     tower = RingingRoomTower(args.room_id, tower_url)
     row_generator = create_row_generator(args)
 
@@ -413,10 +471,22 @@ def console_main(override_args: Optional[List[str]], stop_on_join_tower: bool) -
     except PealSpeedParseError as e:
         sys.exit(f"{e}")
 
-    rhythm = create_rhythm(peal_speed, args.inertia, args.max_bells_in_dataset, args.handstroke_gap,
-                           not args.keep_going)
-    bot = Bot(tower, row_generator, args.use_up_down_in or args.handbell_style,
-              args.stop_at_rounds or args.handbell_style, not args.no_calls, rhythm, user_name=args.name)
+    rhythm = create_rhythm(
+        peal_speed,
+        args.inertia,
+        args.max_bells_in_dataset,
+        args.handstroke_gap,
+        not args.keep_going,
+    )
+    bot = Bot(
+        tower,
+        row_generator,
+        args.use_up_down_in or args.handbell_style,
+        args.stop_at_rounds or args.handbell_style,
+        not args.no_calls,
+        rhythm,
+        user_name=args.name,
+    )
 
     # Catch keyboard interrupts and just print 'Bye!' instead a load of guff
     try:
@@ -427,7 +497,8 @@ def console_main(override_args: Optional[List[str]], stop_on_join_tower: bool) -
     except KeyboardInterrupt:
         print("Bye!")
 
-def main(override_args: Optional[List[str]]=None, stop_on_join_tower: bool=False) -> None:
+
+def main(override_args: Optional[List[str]] = None, stop_on_join_tower: bool = False) -> None:
     """
     The root main function for Wheatley.  If the first argument given to Wheatley is 'server-mode', then
     this will run `server_main` (the main function for running Wheatley on RR's servers) otherwise it will
@@ -445,6 +516,5 @@ def main(override_args: Optional[List[str]]=None, stop_on_join_tower: bool=False
         console_main(unparsed_args, stop_on_join_tower)
 
 
-
 if __name__ == "__main__":
     main()
