@@ -1,27 +1,29 @@
 """ Module to hold DixonoidsGenerator, a class for generating dixonoids. """
 
-from typing import Dict, List
+from typing import Dict, List, Optional
 
-from wheatley.bell import Bell
+from wheatley.aliases import Places, Row
+from wheatley.stroke import Stroke
 
 from .helpers import convert_pn
 from .row_generator import RowGenerator
 
 
 class DixonoidsGenerator(RowGenerator):
-    """ A class to generate rows of dixonoids. """
+    """A class to generate rows of dixonoids."""
 
-    DixonsRules = {
-        0: ["x", "1"],
-        1: ["x", "2"],
-        2: ["x", "4"],
-        4: ["x", "4"]
-    }
+    DixonsRules = {0: ["x", "1"], 1: ["x", "2"], 2: ["x", "4"], 4: ["x", "4"]}
     DefaultBob = {1: ["x", "4"]}
     DefaultSingle = {1: ["x", "1234"]}
 
-    def __init__(self, stage: int, plain_rules: Dict[int, List[str]] = None,
-                 bob_rules: Dict[int, List[str]] = None, single_rules: Dict[int, List[str]] = None):
+    def __init__(
+        self,
+        stage: int,
+        plain_rules: Dict[int, List[str]] = None,
+        bob_rules: Dict[int, List[str]] = None,
+        single_rules: Dict[int, List[str]] = None,
+        start_row: Optional[str] = None,
+    ) -> None:
         """
         Initialises a dixonoid generator.
 
@@ -32,7 +34,7 @@ class DixonoidsGenerator(RowGenerator):
         :param single_rules: Dictionary of leading bell: [handstroke pn, backstroke pn]
                           Only include bells which lead when a single is rung
         """
-        super().__init__(stage)
+        super().__init__(stage, start_row)
 
         if plain_rules is None:
             plain_rules = self.DixonsRules
@@ -47,19 +49,19 @@ class DixonoidsGenerator(RowGenerator):
         self.bob_rules = self._convert_pn_dict(bob_rules)
         self.single_rules = self._convert_pn_dict(single_rules)
 
-    def _gen_row(self, previous_row: List[Bell], is_handstroke: bool, index: int) -> List[Bell]:
+    def _gen_row(self, previous_row: Row, stroke: Stroke, index: int) -> Row:
         leading_bell = previous_row[0].number
-        pn_index = 0 if is_handstroke else 1
+        pn_index = 0 if stroke.is_hand() else 1
 
         if self._has_bob and self.bob_rules.get(leading_bell):
             place_notation = self.bob_rules[leading_bell][pn_index]
 
-            if not is_handstroke:
+            if stroke.is_back():
                 self.reset_calls()
         elif self._has_single and self.single_rules.get(leading_bell):
             place_notation = self.single_rules[leading_bell][pn_index]
 
-            if not is_handstroke:
+            if stroke.is_back():
                 self.reset_calls()
         elif self.plain_rules.get(leading_bell):
             place_notation = self.plain_rules[leading_bell][pn_index]
@@ -69,6 +71,10 @@ class DixonoidsGenerator(RowGenerator):
         row = self.permute(previous_row, place_notation)
         return row
 
+    def summary_string(self) -> str:
+        """Returns a short string summarising the RowGenerator."""
+        return f"dixonoid: {self.plain_rules}"
+
     @staticmethod
-    def _convert_pn_dict(rules: Dict[int, List[str]]) -> Dict[int, List[List[int]]]:
+    def _convert_pn_dict(rules: Dict[int, List[str]]) -> Dict[int, List[Places]]:
         return {key: [convert_pn(pn)[0] for pn in places] for key, places in rules.items()}
