@@ -1,6 +1,6 @@
 """ A module to hold the row generator that generates rows given some place notations. """
 
-from typing import ClassVar, List, Dict
+from typing import ClassVar, List, Dict, Optional
 
 from wheatley.aliases import CallDef, Row, Places
 from wheatley.stroke import Stroke
@@ -10,21 +10,28 @@ from .row_generator import RowGenerator
 
 
 class PlaceNotationGenerator(RowGenerator):
-    """ A row generator to generate rows given a place notation. """
+    """A row generator to generate rows given a place notation."""
 
     # Dict Lead Index: String PlaceNotation
     # 0 for end of the lead
-    DefaultBob: ClassVar[CallDef] = CallDef({0: '14'})
-    DefaultSingle: ClassVar[CallDef] = CallDef({0: '1234'})
+    DEFAULT_BOB: ClassVar[CallDef] = CallDef({0: "14"})
+    DEFAULT_SINGLE: ClassVar[CallDef] = CallDef({0: "1234"})
 
-    def __init__(self, stage: int, method: str, bob: CallDef = None, single: CallDef = None,
-                 start_index: int = 0) -> None:
-        super().__init__(stage)
+    def __init__(
+        self,
+        stage: int,
+        method: str,
+        bob: CallDef = None,
+        single: CallDef = None,
+        start_index: int = 0,
+        start_row: Optional[str] = None,
+    ) -> None:
+        super().__init__(stage, start_row)
 
         if bob is None:
-            bob = PlaceNotationGenerator.DefaultBob
+            bob = PlaceNotationGenerator.DEFAULT_BOB
         if single is None:
-            single = PlaceNotationGenerator.DefaultSingle
+            single = PlaceNotationGenerator.DEFAULT_SINGLE
 
         self.method_pn = convert_pn(method)
         self.lead_len = len(self.method_pn)
@@ -33,7 +40,7 @@ class PlaceNotationGenerator(RowGenerator):
         self.start_index = start_index
 
         def parse_call_dict(unparsed_calls: CallDef) -> Dict[int, List[Places]]:
-            """ Parse a dict of type `int => str` to `int => [PlaceNotation]`. """
+            """Parse a dict of type `int => str` to `int => [PlaceNotation]`."""
             parsed_calls = {}
 
             for i, place_notation_str in unparsed_calls.items():
@@ -53,7 +60,7 @@ class PlaceNotationGenerator(RowGenerator):
         self._generating_call_pn: List[Places] = []
 
     def summary_string(self) -> str:
-        """ Returns a short string summarising the RowGenerator. """
+        """Returns a short string summarising the RowGenerator."""
         return f"place notation '{self.method_pn_string}'"
 
     def _gen_row(self, previous_row: Row, stroke: Stroke, index: int) -> Row:
@@ -80,25 +87,27 @@ class PlaceNotationGenerator(RowGenerator):
         return Stroke.from_index(self.start_index)
 
     @staticmethod
-    def grandsire(stage: int) -> RowGenerator:
-        """ Generates Grandsire on a given stage. """
+    def grandsire(stage: int, start_row: Optional[str] = None) -> RowGenerator:
+        """Generates Grandsire on a given stage."""
         stage_bell = convert_to_bell_string(stage)
 
-        cross_notation = stage_bell if stage % 2 else '-'
+        cross_notation = stage_bell if stage % 2 else "-"
 
         main_body = ["1" if i % 2 else cross_notation for i in range(2 * stage)]
         main_body[0] = "3"
         notation = ".".join(main_body)
 
-        return PlaceNotationGenerator(stage, notation, bob=CallDef({-1: "3"}), single=CallDef({-1: "3.123"}))
+        return PlaceNotationGenerator(
+            stage, notation, bob=CallDef({-1: "3"}), single=CallDef({-1: "3.123"}), start_row=start_row
+        )
 
     @staticmethod
-    def stedman(stage: int) -> RowGenerator:
-        """ Generates Stedman on a given stage (even bell Stedman will cause an exception). """
+    def stedman(stage: int, start_row: Optional[str] = None) -> RowGenerator:
+        """Generates Stedman on a given stage (even bell Stedman will cause an exception)."""
         assert stage % 2 == 1
 
         if stage == 5:
-            return PlaceNotationGenerator.stedman_doubles()
+            return PlaceNotationGenerator.stedman_doubles(start_row)
 
         stage_bell = convert_to_bell_string(stage)
         stage_bell_1 = convert_to_bell_string(stage - 1)
@@ -106,13 +115,24 @@ class PlaceNotationGenerator(RowGenerator):
 
         notation = f"3.1.{stage_bell}.3.1.3.1.3.{stage_bell}.1.3.1"
 
-        return PlaceNotationGenerator(stage, notation, bob=CallDef({3: stage_bell_2, 9: stage_bell_2}),
-                                      single=CallDef({3: f"{stage_bell_2}{stage_bell_1}{stage_bell}",
-                                              9: f"{stage_bell_2}{stage_bell_1}{stage_bell}"}))
+        return PlaceNotationGenerator(
+            stage,
+            notation,
+            bob=CallDef({3: stage_bell_2, 9: stage_bell_2}),
+            single=CallDef(
+                {
+                    3: f"{stage_bell_2}{stage_bell_1}{stage_bell}",
+                    9: f"{stage_bell_2}{stage_bell_1}{stage_bell}",
+                }
+            ),
+            start_row=start_row,
+        )
 
     @staticmethod
-    def stedman_doubles() -> RowGenerator:
-        """ Generates Stedman on a given stage (even bell Stedman will cause an exception). """
+    def stedman_doubles(start_row: Optional[str] = None) -> RowGenerator:
+        """Generates Stedman on a given stage (even bell Stedman will cause an exception)."""
         notation = "3.1.5.3.1.3.1.3.5.1.3.1"
 
-        return PlaceNotationGenerator(5, notation, bob=CallDef({}), single=CallDef({6: "345", 12: "145"}))
+        return PlaceNotationGenerator(
+            5, notation, bob=CallDef({}), single=CallDef({6: "345", 12: "145"}), start_row=start_row
+        )
